@@ -237,12 +237,12 @@ retrysrch:
                 }
             }
         } else if (userdnattr) {
-            sprintf(searchbase, "%s=%s, %s", userdnattr, login, userbasedn);
+            snprintf(filter,8192,"%s=%s",userdnattr,login);
 
 retrydnattr:
             if (show_debug_messages)
-                fprintf(stderr, "searchbase '%s'\n", searchbase);
-            rc = ldap_search_s(ld, searchbase, searchscope, NULL, NULL, 0, &res);
+                fprintf(stderr, "searchbase '%s'\n", userbasedn);
+            rc = ldap_search_s(ld, userbasedn, searchscope, filter, NULL, 0, &res);
         }
         if (rc == LDAP_SUCCESS) {
             entry = ldap_first_entry(ld, res);
