@@ -33,19 +33,23 @@
  *
  * Copyright (c) 2003, Robert Collins <robertc@squid-cache.org>
  */
+
+#include "config.h"
+
 #ifndef COMPOSITEPOOLNODE_H
 #define COMPOSITEPOOLNODE_H
 
-#if USE_DELAY_POOLS
+#if DELAY_POOLS
 #include "squid.h"
-#include "auth/UserRequest.h"
 #include "DelayPools.h"
 #include "DelayIdComposite.h"
 #include "CommRead.h"
-#include "ip/Address.h"
+#include "ip/IpAddress.h"
 
 class StoreEntry;
 
+class AuthUserRequest;
+
 /// \ingroup DelayPoolsAPI
 class CompositePoolNode : public RefCountable, public Updateable
 {
@@ -72,10 +76,8 @@ public:
     public:
         CompositeSelectionDetails() {}
 
-        Ip::Address src_addr;
-#if USE_AUTH
-        AuthUserRequest::Pointer user;
-#endif
+        IpAddress src_addr;
+        AuthUserRequest *user;
         String tag;
     };
 
@@ -84,5 +86,5 @@ protected:
     DeferredReadManager deferredReads;
 };
 
-#endif /* USE_DELAY_POOLS */
+#endif /* DELAY_POOLS */
 #endif /* COMPOSITEPOOLNODE_H */
