@@ -37,10 +37,10 @@
  * Copyright (c) 2003, Robert Collins <robertc@squid-cache.org>
  */
 
-#include "squid.h"
+#include "config.h"
 
-#if USE_DELAY_POOLS
-#include "squid-old.h"
+#if DELAY_POOLS
+#include "squid.h"
 #include "DelayConfig.h"
 #include "ConfigParser.h"
 #include "DelayPools.h"
@@ -52,7 +52,7 @@
 void
 DelayConfig::parsePoolCount()
 {
-    unsigned short pools_;
+    u_short pools_;
     ConfigParser::ParseUShort(&pools_);
     DelayPools::pools(pools_);
 }
@@ -60,7 +60,7 @@ DelayConfig::parsePoolCount()
 void
 DelayConfig::parsePoolClass()
 {
-    unsigned short pool;
+    ushort pool;
 
     ConfigParser::ParseUShort(&pool);
 
@@ -69,7 +69,7 @@ DelayConfig::parsePoolClass()
         return;
     }
 
-    unsigned short delay_class_;
+    ushort delay_class_;
     ConfigParser::ParseUShort(&delay_class_);
 
     if (delay_class_ < 1 || delay_class_ > 5) {
@@ -77,7 +77,7 @@ DelayConfig::parsePoolClass()
         return;
     }
 
-    --pool;
+    pool--;
 
     DelayPools::delay_data[pool].createPool(delay_class_);
 }
@@ -85,7 +85,7 @@ DelayConfig::parsePoolClass()
 void
 DelayConfig::parsePoolRates()
 {
-    unsigned short pool;
+    ushort pool;
     ConfigParser::ParseUShort(&pool);
 
     if (pool < 1 || pool > DelayPools::pools()) {
@@ -93,7 +93,7 @@ DelayConfig::parsePoolRates()
         return;
     }
 
-    --pool;
+    pool--;
 
     if (!DelayPools::delay_data[pool].theComposite().getRaw()) {
         debugs(3, 0, "parse_delay_pool_rates: Ignoring pool " << pool + 1 << " attempt to set rates with class not set");
@@ -106,7 +106,7 @@ DelayConfig::parsePoolRates()
 void
 DelayConfig::parsePoolAccess(ConfigParser &parser)
 {
-    unsigned short pool;
+    ushort pool;
 
     ConfigParser::ParseUShort(&pool);
 
@@ -138,8 +138,8 @@ DelayConfig::dumpPoolCount(StoreEntry * entry, const char *name) const
 
     storeAppendPrintf(entry, "%s %d\n", name, DelayPools::pools());
 
-    for (i = 0; i < DelayPools::pools(); ++i)
+    for (i = 0; i < DelayPools::pools(); i++)
         DelayPools::delay_data[i].dump (entry, i);
 }
 
-#endif /* USE_DELAY_POOLS */
+#endif
