@@ -34,9 +34,11 @@
 #ifndef __WIN32_AIO_H__
 #define __WIN32_AIO_H__
 
+#include "config.h"
+
 #if USE_DISKIO_AIO
 
-#if _SQUID_CYGWIN_
+#ifdef _SQUID_CYGWIN_
 #include "squid_windows.h"
 #endif
 
