@@ -34,41 +34,43 @@
  */
 
 #include "squid.h"
-#include "HttpBody.h"
 #include "MemBuf.h"
 
 
-HttpBody::HttpBody() : mb(new MemBuf)
-{}
-
-HttpBody::~HttpBody()
+void
+httpBodyInit(HttpBody * body)
 {
-    delete mb;
+    body->mb = new MemBuf;
 }
 
 void
-HttpBody::clear()
+httpBodyClean(HttpBody * body)
 {
-    mb->clean();
+    assert(body);
+
+    if (!body->mb->isNull())
+        body->mb->clean();
+
+    delete body->mb;
+
+    body->mb = NULL;
 }
 
 /* set body by absorbing mb */
 void
-HttpBody::setMb(MemBuf * mb_)
+httpBodySet(HttpBody * body, MemBuf * mb)
 {
-    delete mb;
-    /* note: protection against assign-to-self is not needed
-     * as MemBuf doesn't have a copy-constructor. If such a constructor
-     * is ever added, add such protection here.
-     */
-    mb = mb_;		/* absorb */
+    assert(body);
+    assert(body->mb->isNull());
+    delete body->mb;
+    body->mb = mb;		/* absorb */
 }
 
 void
-HttpBody::packInto(Packer * p) const
+httpBodyPackInto(const HttpBody * body, Packer * p)
 {
-    assert(p);
+    assert(body && p);
 
-    if (mb->contentSize())
-        packerAppend(p, mb->content(), mb->contentSize());
+    if (body->mb->contentSize())
+        packerAppend(p, body->mb->content(), body->mb->contentSize());
 }
