@@ -33,6 +33,9 @@ public:
 
     virtual Initiate *makeXactLauncher(HttpMsg *virginHeader, HttpRequest *virginCause) = 0;
 
+    typedef void Callback(void *data, Pointer &service);
+    void callWhenReady(Callback *cb, void *data);
+
     bool wants(const ServiceFilter &filter) const;
 
     // the methods below can only be called on an up() service
