@@ -1,3 +1,4 @@
+
 /*
  * $Id$
  *
@@ -36,7 +37,7 @@
  * They DO NOT perform access control or auditing.
  * See acl.c for access control and client_side.c for auditing */
 
-#include "squid-old.h"
+#include "squid.h"
 #include "acl/Acl.h"
 #include "acl/FilledChecklist.h"
 #include "client_side.h"
@@ -56,11 +57,11 @@ authenticateActiveSchemeCount(void)
 {
     int rv = 0;
 
-    for (Auth::ConfigVector::iterator i = Auth::TheConfig.begin(); i != Auth::TheConfig.end(); ++i)
+    for (authConfig::iterator i = Config.authConfiguration.begin(); i != Config.authConfiguration.end(); ++i)
         if ((*i)->configured())
             ++rv;
 
-    debugs(29, 9, HERE << rv << " active.");
+    debugs(29, 9, "authenticateActiveSchemeCount: " << rv << " active.");
 
     return rv;
 }
@@ -68,84 +69,119 @@ authenticateActiveSchemeCount(void)
 int
 authenticateSchemeCount(void)
 {
-    int rv = Auth::Scheme::GetSchemes().size();
+    int rv = AuthScheme::Schemes().size();
 
-    debugs(29, 9, HERE << rv << " active.");
+    debugs(29, 9, "authenticateSchemeCount: " << rv << " active.");
 
     return rv;
 }
 
 static void
-authenticateRegisterWithCacheManager(Auth::ConfigVector * config)
+authenticateRegisterWithCacheManager(authConfig * config)
 {
-    for (Auth::ConfigVector::iterator i = config->begin(); i != config->end(); ++i) {
-        Auth::Config *scheme = *i;
+    for (authConfig::iterator i = config->begin(); i != config->end(); ++i) {
+        AuthConfig *scheme = *i;
         scheme->registerWithCacheManager();
     }
 }
 
 void
-authenticateInit(Auth::ConfigVector * config)
+authenticateInit(authConfig * config)
 {
-    /* Do this first to clear memory and remove dead state on a reconfigure */
-    if (proxy_auth_username_cache)
-        Auth::User::CachedACLsReset();
+    for (authConfig::iterator i = config->begin(); i != config->end(); ++i) {
+        AuthConfig *scheme = *i;
 
-    /* If we do not have any auth config state to create stop now. */
-    if (!config)
-        return;
-
-    for (Auth::ConfigVector::iterator i = config->begin(); i != config->end(); ++i) {
-        Auth::Config *schemeCfg = *i;
-
-        if (schemeCfg->configured())
-            schemeCfg->init(schemeCfg);
+        if (scheme->configured())
+            scheme->init(scheme);
     }
 
     if (!proxy_auth_username_cache)
-        Auth::User::cacheInit();
+        AuthUser::cacheInit();
+    else
+        AuthUser::CachedACLsReset();
 
-    authenticateRegisterWithCacheManager(config);
+    authenticateRegisterWithCacheManager(&Config.authConfiguration);
 }
 
 void
-authenticateRotate(void)
+authenticateShutdown(void)
 {
-    for (Auth::ConfigVector::iterator i = Auth::TheConfig.begin(); i != Auth::TheConfig.end(); ++i)
-        if ((*i)->configured())
-            (*i)->rotateHelpers();
-}
+    debugs(29, 2, "authenticateShutdown: shutting down auth schemes");
+    /* free the cache if we are shutting down */
 
-void
-authenticateReset(void)
-{
-    debugs(29, 2, HERE << "Reset authentication State.");
-
-    /* free all username cache entries */
-    hash_first(proxy_auth_username_cache);
-    AuthUserHashPointer *usernamehash;
-    while ((usernamehash = ((AuthUserHashPointer *) hash_next(proxy_auth_username_cache)))) {
-        debugs(29, 5, HERE << "Clearing entry for user: " << usernamehash->user()->username());
-        hash_remove_link(proxy_auth_username_cache, (hash_link *)usernamehash);
-        delete usernamehash;
+    if (shutting_down) {
+        hashFreeItems(proxy_auth_username_cache, AuthUserHashPointer::removeFromCache);
+        AuthScheme::FreeAll();
+    } else {
+        for (AuthScheme::const_iterator i = AuthScheme::Schemes().begin(); i != AuthScheme::Schemes().end(); ++i)
+            (*i)->done();
     }
-
-    /* schedule shutdown of the helpers */
-    authenticateRotate();
-
-    /* free current global config details too. */
-    Auth::TheConfig.clean();
 }
 
-AuthUserHashPointer::AuthUserHashPointer(Auth::User::Pointer anAuth_user):
+/**
+ \retval 0 not in use
+ \retval ? in use
+ */
+int
+authenticateAuthUserInuse(AuthUser * auth_user)
+{
+    assert(auth_user != NULL);
+    return auth_user->references;
+}
+
+void
+authenticateAuthUserMerge(AuthUser * from, AuthUser * to)
+{
+    to->absorb (from);
+}
+
+/**
+ * Cleans all config-dependent data from the auth_user cache.
+ \note It DOES NOT Flush the user cache.
+ */
+void
+authenticateUserCacheRestart(void)
+{
+    AuthUserHashPointer *usernamehash;
+    AuthUser *auth_user;
+    debugs(29, 3, HERE << "Clearing config dependent cache data.");
+    hash_first(proxy_auth_username_cache);
+
+    while ((usernamehash = ((AuthUserHashPointer *) hash_next(proxy_auth_username_cache)))) {
+        auth_user = usernamehash->user();
+        debugs(29, 5, "authenticateUserCacheRestat: Clearing cache ACL results for user: " << auth_user->username());
+    }
+}
+
+
+void
+AuthUserHashPointer::removeFromCache(void *usernamehash_p)
+{
+    AuthUserHashPointer *usernamehash = static_cast<AuthUserHashPointer *>(usernamehash_p);
+    AuthUser *auth_user = usernamehash->auth_user;
+
+    if ((authenticateAuthUserInuse(auth_user) - 1))
+        debugs(29, 1, "AuthUserHashPointer::removeFromCache: entry in use - not freeing");
+
+    auth_user->unlock();
+
+    /** \todo change behaviour - we remove from the auth user list here, and then unlock, and the
+     * delete ourselves.
+     */
+}
+
+AuthUserHashPointer::AuthUserHashPointer(AuthUser * anAuth_user):
         auth_user(anAuth_user)
 {
     key = (void *)anAuth_user->username();
     next = NULL;
     hash_join(proxy_auth_username_cache, (hash_link *) this);
+
+    /** lock for presence in the cache */
+    auth_user->lock();
 }
 
-Auth::User::Pointer
+AuthUser *
 AuthUserHashPointer::user() const
 {
     return auth_user;
