@@ -1,3 +1,4 @@
+
 /*
  * $Id$
  *
@@ -34,7 +35,6 @@
 #define SQUID_DIGESTSCHEME_H
 
 #include "auth/Scheme.h"
-#include "auth/digest/auth_digest.h"
 
 /// \ingroup AuthSchemeAPI
 /// \ingroup AuthAPI
@@ -42,28 +42,20 @@ class digestScheme : public AuthScheme
 {
 
 public:
-    static AuthScheme::Pointer GetInstance();
-    digestScheme() {};
+    static AuthScheme &GetInstance();
+    digestScheme();
     virtual ~digestScheme() {}
 
     /* per scheme */
     virtual char const *type () const;
     virtual void done();
     virtual AuthConfig *createConfig();
-
     /* Not implemented */
     digestScheme (digestScheme const &);
     digestScheme &operator=(digestScheme const &);
 
 private:
-    static AuthScheme::Pointer _instance;
-
-    /**
-     * Remove all cached user credentials from circulation.
-     * Intended for use during shutdown procedure.
-     * After calling this all newly received credentials must be re-authenticated.
-     */
-    static void PurgeCredentialsCache(void);
+    static digestScheme *_instance;
 };
 
 #endif /* SQUID_DIGESTSCHEME_H */
