@@ -35,7 +35,7 @@
 #ifndef SQUID_CLIENTSIDEREQUEST_H
 #define SQUID_CLIENTSIDEREQUEST_H
 
-#include "squid-old.h"
+#include "squid.h"
 #include "HttpHeader.h"
 #include "clientStream.h"
 #include "client_side.h"
@@ -72,7 +72,7 @@ public:
 #if USE_ADAPTATION
     void *toCbdata() { return this; }
 #endif
-    ClientHttpRequest(ConnStateData *csd);
+    ClientHttpRequest(ConnStateData *);
     ~ClientHttpRequest();
     /* Not implemented - present to prevent synthetic operations */
     ClientHttpRequest(ClientHttpRequest const &);
@@ -95,12 +95,6 @@ public:
 
     _SQUID_INLINE_ ConnStateData * getConn() const;
     _SQUID_INLINE_ void setConn(ConnStateData *);
-
-    /** Details of the client socket which produced us.
-     * Treat as read-only for the lifetime of this HTTP request.
-     */
-    Comm::ConnectionPointer clientConnection;
-
     HttpRequest *request;		/* Parsed URL ... */
     char *uri;
     char *log_uri;
@@ -173,14 +167,12 @@ public:
     void startAdaptation(const Adaptation::ServiceGroupPointer &g);
 
     // private but exposed for ClientRequestContext
-    void handleAdaptationFailure(int errDetail, bool bypassable = false);
+    void handleAdaptationFailure(bool bypassable = false);
 
 private:
     // Adaptation::Initiator API
-    virtual void noteAdaptationAnswer(const Adaptation::Answer &answer);
-    void handleAdaptedHeader(HttpMsg *msg);
-    void handleAdaptationBlock(const Adaptation::Answer &answer);
-    virtual void noteAdaptationAclCheckDone(Adaptation::ServiceGroupPointer group);
+    virtual void noteAdaptationAnswer(HttpMsg *message);
+    virtual void noteAdaptationQueryAbort(bool final);
 
     // BodyConsumer API, called by BodyPipe
     virtual void noteMoreBodyDataAvailable(BodyPipe::Pointer);
@@ -188,8 +180,6 @@ private:
     virtual void noteBodyProducerAborted(BodyPipe::Pointer);
 
     void endRequestSatisfaction();
-    /// called by StoreEntry when it has more buffer space available
-    void resumeBodyStorage();
 
 private:
     CbcPointer<Adaptation::Initiate> virginHeadSource;
@@ -213,7 +203,7 @@ SQUIDCEXTERN void redirectStart(ClientHttpRequest *, RH *, void *);
 
 SQUIDCEXTERN void tunnelStart(ClientHttpRequest *, int64_t *, int *);
 
-#if _USE_INLINE_
+#ifdef _USE_INLINE_
 #include "Store.h"
 #include "client_side_request.cci"
 #endif
