@@ -32,38 +32,26 @@
  *
  */
 
-#include "squid-old.h"
-#include "base/AsyncCbdataCalls.h"
-#include "comm.h"
-#include "comm/Connection.h"
-#include "comm/Write.h"
+#include "squid.h"
 #include "helper.h"
-#include "format/Quoting.h"
-#include "MemBuf.h"
 #include "SquidMath.h"
 #include "SquidTime.h"
 #include "Store.h"
+#include "comm.h"
+#include "MemBuf.h"
 #include "wordlist.h"
 
 #define HELPER_MAX_ARGS 64
 
-
-/** Initial Squid input buffer size. Helper responses may exceed this, and
- * Squid will grow the input buffer as needed, up to ReadBufMaxSize.
- */
-const size_t ReadBufMinSize(4*1024);
-
-/** Maximum safe size of a helper-to-Squid response message plus one.
- * Squid will warn and close the stream if a helper sends a too-big response.
- * ssl_crtd helper is known to produce responses of at least 10KB in size.
- * Some undocumented helpers are known to produce responses exceeding 8KB.
- */
-const size_t ReadBufMaxSize(32*1024);
+/* size of helper read buffer (maximum?). no reason given for this size */
+/* though it has been seen to be too short for some requests */
+/* it is dynamic, so increasng should not have side effects */
+#define BUF_8KB	8192
 
 static IOCB helperHandleRead;
 static IOCB helperStatefulHandleRead;
-static void helperServerFree(helper_server *srv);
-static void helperStatefulServerFree(helper_stateful_server *srv);
+static PF helperServerFree;
+static PF helperStatefulServerFree;
 static void Enqueue(helper * hlp, helper_request *);
 static helper_request *Dequeue(helper * hlp);
 static helper_stateful_request *StatefulDequeue(statefulhelper * hlp);
@@ -80,67 +68,11 @@ static void StatefulEnqueue(statefulhelper * hlp, helper_stateful_request * r);
 static bool helperStartStats(StoreEntry *sentry, void *hlp, const char *label);
 
 
-CBDATA_CLASS_INIT(helper);
+CBDATA_TYPE(helper);
 CBDATA_TYPE(helper_server);
-CBDATA_CLASS_INIT(statefulhelper);
+CBDATA_TYPE(statefulhelper);
 CBDATA_TYPE(helper_stateful_server);
 
-void
-HelperServerBase::closePipesSafely()
-{
-#if _SQUID_MSWIN_
-    int no = index + 1;
-
-    shutdown(writePipe->fd, SD_BOTH);
-#endif
-
-    flags.closing = 1;
-    if (readPipe->fd == writePipe->fd)
-        readPipe->fd = -1;
-    else
-        readPipe->close();
-    writePipe->close();
-
-#if _SQUID_MSWIN_
-    if (hIpc) {
-        if (WaitForSingleObject(hIpc, 5000) != WAIT_OBJECT_0) {
-            getCurrentTime();
-            debugs(84, DBG_IMPORTANT, "WARNING: " << hlp->id_name <<
-                   " #" << no << " (" << hlp->cmdline->key << "," <<
-                   (long int)pid << ") didn't exit in 5 seconds");
-        }
-        CloseHandle(hIpc);
-    }
-#endif
-}
-
-void
-HelperServerBase::closeWritePipeSafely()
-{
-#if _SQUID_MSWIN_
-    int no = index + 1;
-
-    shutdown(writePipe->fd, (readPipe->fd == writePipe->fd ? SD_BOTH : SD_SEND));
-#endif
-
-    flags.closing = 1;
-    if (readPipe->fd == writePipe->fd)
-        readPipe->fd = -1;
-    writePipe->close();
-
-#if _SQUID_MSWIN_
-    if (hIpc) {
-        if (WaitForSingleObject(hIpc, 5000) != WAIT_OBJECT_0) {
-            getCurrentTime();
-            debugs(84, DBG_IMPORTANT, "WARNING: " << hlp->id_name <<
-                   " #" << no << " (" << hlp->cmdline->key << "," <<
-                   (long int)pid << ") didn't exit in 5 seconds");
-        }
-        CloseHandle(hIpc);
-    }
-#endif
-}
-
 void
 helperOpenServers(helper * hlp)
 {
@@ -169,10 +101,10 @@ helperOpenServers(helper * hlp)
     else
         shortname = xstrdup(progname);
 
-    /* figure out how many new child are actually needed. */
-    int need_new = hlp->childs.needNew();
+    /* dont ever start more than hlp->n_to_start processes. */
+    int need_new = hlp->n_to_start - hlp->n_active;
 
-    debugs(84, 1, "helperOpenServers: Starting " << need_new << "/" << hlp->childs.n_max << " '" << shortname << "' processes");
+    debugs(84, 1, "helperOpenServers: Starting " << need_new << "/" << hlp->n_to_start << " '" << shortname << "' processes");
 
     if (need_new < 1) {
         debugs(84, 1, "helperOpenServers: No '" << shortname << "' processes needed.");
@@ -182,20 +114,16 @@ helperOpenServers(helper * hlp)
 
     snprintf(procname, strlen(shortname) + 3, "(%s)", shortname);
 
-    args[nargs] = procname;
-    ++nargs;
+    args[nargs++] = procname;
 
-    for (w = hlp->cmdline->next; w && nargs < HELPER_MAX_ARGS; w = w->next) {
-        args[nargs] = w->key;
-        ++nargs;
-    }
+    for (w = hlp->cmdline->next; w && nargs < HELPER_MAX_ARGS; w = w->next)
+        args[nargs++] = w->key;
 
-    args[nargs] = NULL;
-    ++nargs;
+    args[nargs++] = NULL;
 
     assert(nargs <= HELPER_MAX_ARGS);
 
-    for (k = 0; k < need_new; ++k) {
+    for (k = 0; k < need_new; k++) {
         getCurrentTime();
         rfd = wfd = -1;
         pid = ipcCreate(hlp->ipc_type,
@@ -212,22 +140,20 @@ helperOpenServers(helper * hlp)
             continue;
         }
 
-        ++ hlp->childs.n_running;
-        ++ hlp->childs.n_active;
+        hlp->n_running++;
+        hlp->n_active++;
         CBDATA_INIT_TYPE(helper_server);
         srv = cbdataAlloc(helper_server);
         srv->hIpc = hIpc;
         srv->pid = pid;
         srv->index = k;
         srv->addr = hlp->addr;
-        srv->readPipe = new Comm::Connection;
-        srv->readPipe->fd = rfd;
-        srv->writePipe = new Comm::Connection;
-        srv->writePipe->fd = wfd;
-        srv->rbuf = (char *)memAllocBuf(ReadBufMinSize, &srv->rbuf_sz);
+        srv->rfd = rfd;
+        srv->wfd = wfd;
+        srv->rbuf = (char *)memAllocBuf(BUF_8KB, &srv->rbuf_sz);
         srv->wqueue = new MemBuf;
         srv->roffset = 0;
-        srv->requests = (helper_request **)xcalloc(hlp->childs.concurrency ? hlp->childs.concurrency : 1, sizeof(*srv->requests));
+        srv->requests = (helper_request **)xcalloc(hlp->concurrency ? hlp->concurrency : 1, sizeof(*srv->requests));
         srv->parent = cbdataReference(hlp);
         dlinkAddTail(srv, &srv->link, &hlp->servers);
 
@@ -246,12 +172,9 @@ helperOpenServers(helper * hlp)
         if (wfd != rfd)
             commSetNonBlocking(wfd);
 
-        AsyncCall::Pointer closeCall = asyncCall(5,4, "helperServerFree", cbdataDialer(helperServerFree, srv));
-        comm_add_close_handler(rfd, closeCall);
+        comm_add_close_handler(rfd, helperServerFree, srv);
 
-        AsyncCall::Pointer call = commCbCall(5,4, "helperHandleRead",
-                                             CommIoCbPtrFun(helperHandleRead, srv));
-        comm_read(srv->readPipe, srv->rbuf, srv->rbuf_sz - 1, call);
+        comm_read(srv->rfd, srv->rbuf, srv->rbuf_sz - 1, helperHandleRead, srv);
     }
 
     hlp->last_restart = squid_curtime;
@@ -276,9 +199,6 @@ helperStatefulOpenServers(statefulhelper * hlp)
     if (hlp->cmdline == NULL)
         return;
 
-    if (hlp->childs.concurrency)
-        debugs(84, 0, "ERROR: concurrency= is not yet supported for stateful helpers ('" << hlp->cmdline << "')");
-
     char *progname = hlp->cmdline->key;
 
     char *s;
@@ -287,10 +207,11 @@ helperStatefulOpenServers(statefulhelper * hlp)
     else
         shortname = xstrdup(progname);
 
-    /* figure out haw mant new helpers are needed. */
-    int need_new = hlp->childs.needNew();
+    /* dont ever start more than hlp->n_to_start processes. */
+    /* n_active are the helpers which have not been shut down. */
+    int need_new = hlp->n_to_start - hlp->n_active;
 
-    debugs(84, 1, "helperOpenServers: Starting " << need_new << "/" << hlp->childs.n_max << " '" << shortname << "' processes");
+    debugs(84, 1, "helperOpenServers: Starting " << need_new << "/" << hlp->n_to_start << " '" << shortname << "' processes");
 
     if (need_new < 1) {
         debugs(84, 1, "helperStatefulOpenServers: No '" << shortname << "' processes needed.");
@@ -300,20 +221,16 @@ helperStatefulOpenServers(statefulhelper * hlp)
 
     snprintf(procname, strlen(shortname) + 3, "(%s)", shortname);
 
-    args[nargs] = procname;
-    ++nargs;
+    args[nargs++] = procname;
 
-    for (wordlist *w = hlp->cmdline->next; w && nargs < HELPER_MAX_ARGS; w = w->next) {
-        args[nargs] = w->key;
-        ++nargs;
-    }
+    for (wordlist *w = hlp->cmdline->next; w && nargs < HELPER_MAX_ARGS; w = w->next)
+        args[nargs++] = w->key;
 
-    args[nargs] = NULL;
-    ++nargs;
+    args[nargs++] = NULL;
 
     assert(nargs <= HELPER_MAX_ARGS);
 
-    for (int k = 0; k < need_new; ++k) {
+    for (int k = 0; k < need_new; k++) {
         getCurrentTime();
         int rfd = -1;
         int wfd = -1;
@@ -332,8 +249,8 @@ helperStatefulOpenServers(statefulhelper * hlp)
             continue;
         }
 
-        ++ hlp->childs.n_running;
-        ++ hlp->childs.n_active;
+        hlp->n_running++;
+        hlp->n_active++;
         CBDATA_INIT_TYPE(helper_stateful_server);
         helper_stateful_server *srv = cbdataAlloc(helper_stateful_server);
         srv->hIpc = hIpc;
@@ -343,11 +260,9 @@ helperStatefulOpenServers(statefulhelper * hlp)
         srv->stats.releases = 0;
         srv->index = k;
         srv->addr = hlp->addr;
-        srv->readPipe = new Comm::Connection;
-        srv->readPipe->fd = rfd;
-        srv->writePipe = new Comm::Connection;
-        srv->writePipe->fd = wfd;
-        srv->rbuf = (char *)memAllocBuf(ReadBufMinSize, &srv->rbuf_sz);
+        srv->rfd = rfd;
+        srv->wfd = wfd;
+        srv->rbuf = (char *)memAllocBuf(BUF_8KB, &srv->rbuf_sz);
         srv->roffset = 0;
         srv->parent = cbdataReference(hlp);
 
@@ -371,12 +286,9 @@ helperStatefulOpenServers(statefulhelper * hlp)
         if (wfd != rfd)
             commSetNonBlocking(wfd);
 
-        AsyncCall::Pointer closeCall = asyncCall(5,4, "helperStatefulServerFree", cbdataDialer(helperStatefulServerFree, srv));
-        comm_add_close_handler(rfd, closeCall);
+        comm_add_close_handler(rfd, helperStatefulServerFree, srv);
 
-        AsyncCall::Pointer call = commCbCall(5,4, "helperStatefulHandleRead",
-                                             CommIoCbPtrFun(helperStatefulHandleRead, srv));
-        comm_read(srv->readPipe, srv->rbuf, srv->rbuf_sz - 1, call);
+        comm_read(srv->rfd, srv->rbuf, srv->rbuf_sz - 1, helperStatefulHandleRead, srv);
     }
 
     hlp->last_restart = squid_curtime;
@@ -464,7 +376,7 @@ helperStatefulReleaseServer(helper_stateful_server * srv)
     if (!srv->flags.reserved)
         return;
 
-    ++ srv->stats.releases;
+    srv->stats.releases++;
 
     srv->flags.reserved = 0;
     if (srv->parent->OnEmptyQueue != NULL && srv->data)
@@ -492,7 +404,7 @@ helperStats(StoreEntry * sentry, helper * hlp, const char *label)
     storeAppendPrintf(sentry, "program: %s\n",
                       hlp->cmdline->key);
     storeAppendPrintf(sentry, "number active: %d of %d (%d shutting down)\n",
-                      hlp->childs.n_active, hlp->childs.n_max, (hlp->childs.n_running - hlp->childs.n_active) );
+                      hlp->n_active, hlp->n_to_start, (hlp->n_running - hlp->n_active) );
     storeAppendPrintf(sentry, "requests sent: %d\n",
                       hlp->stats.requests);
     storeAppendPrintf(sentry, "replies received: %d\n",
@@ -517,7 +429,7 @@ helperStats(StoreEntry * sentry, helper * hlp, const char *label)
         double tt = 0.001 * (srv->requests[0] ? tvSubMsec(srv->requests[0]->dispatch_time, current_time) : tvSubMsec(srv->dispatch_time, srv->answer_time));
         storeAppendPrintf(sentry, "%7d\t%7d\t%7d\t%11d\t%c%c%c%c\t%7.3f\t%7d\t%s\n",
                           srv->index + 1,
-                          srv->readPipe->fd,
+                          srv->rfd,
                           srv->pid,
                           srv->stats.uses,
                           srv->stats.pending ? 'B' : ' ',
@@ -526,7 +438,7 @@ helperStats(StoreEntry * sentry, helper * hlp, const char *label)
                           srv->flags.shutdown ? 'S' : ' ',
                           tt < 0.0 ? 0.0 : tt,
                           (int) srv->roffset,
-                          srv->requests[0] ? Format::QuoteMimeBlob(srv->requests[0]->buf) : "(none)");
+                          srv->requests[0] ? log_quote(srv->requests[0]->buf) : "(none)");
     }
 
     storeAppendPrintf(sentry, "\nFlags key:\n\n");
@@ -545,7 +457,7 @@ helperStatefulStats(StoreEntry * sentry, statefulhelper * hlp, const char *label
     storeAppendPrintf(sentry, "program: %s\n",
                       hlp->cmdline->key);
     storeAppendPrintf(sentry, "number active: %d of %d (%d shutting down)\n",
-                      hlp->childs.n_active, hlp->childs.n_max, (hlp->childs.n_running - hlp->childs.n_active) );
+                      hlp->n_active, hlp->n_to_start, (hlp->n_running - hlp->n_active) );
     storeAppendPrintf(sentry, "requests sent: %d\n",
                       hlp->stats.requests);
     storeAppendPrintf(sentry, "replies received: %d\n",
@@ -570,7 +482,7 @@ helperStatefulStats(StoreEntry * sentry, statefulhelper * hlp, const char *label
         double tt = 0.001 * tvSubMsec(srv->dispatch_time, srv->flags.busy ? current_time : srv->answer_time);
         storeAppendPrintf(sentry, "%7d\t%7d\t%7d\t%11d\t%c%c%c%c%c\t%7.3f\t%7d\t%s\n",
                           srv->index + 1,
-                          srv->readPipe->fd,
+                          srv->rfd,
                           srv->pid,
                           srv->stats.uses,
                           srv->flags.busy ? 'B' : ' ',
@@ -580,7 +492,7 @@ helperStatefulStats(StoreEntry * sentry, statefulhelper * hlp, const char *label
                           srv->request ? (srv->request->placeholder ? 'P' : ' ') : ' ',
                                   tt < 0.0 ? 0.0 : tt,
                                   (int) srv->roffset,
-                                  srv->request ? Format::QuoteMimeBlob(srv->request->buf) : "(none)");
+                                  srv->request ? log_quote(srv->request->buf) : "(none)");
     }
 
     storeAppendPrintf(sentry, "\nFlags key:\n\n");
@@ -595,6 +507,12 @@ void
 helperShutdown(helper * hlp)
 {
     dlink_node *link = hlp->servers.head;
+#ifdef _SQUID_MSWIN_
+
+    HANDLE hIpc;
+    pid_t pid;
+    int no;
+#endif
 
     while (link) {
         helper_server *srv;
@@ -606,8 +524,8 @@ helperShutdown(helper * hlp)
             continue;
         }
 
-        assert(hlp->childs.n_active > 0);
-        -- hlp->childs.n_active;
+        hlp->n_active--;
+        assert(hlp->n_active >= 0);
         srv->flags.shutdown = 1;	/* request it to shut itself down */
 
         if (srv->flags.closing) {
@@ -620,11 +538,36 @@ helperShutdown(helper * hlp)
             continue;
         }
 
+        srv->flags.closing = 1;
+#ifdef _SQUID_MSWIN_
+
+        hIpc = srv->hIpc;
+        pid = srv->pid;
+        no = srv->index + 1;
+        shutdown(srv->wfd, SD_BOTH);
+#endif
+
         debugs(84, 3, "helperShutdown: " << hlp->id_name << " #" << srv->index + 1 << " shutting down.");
         /* the rest of the details is dealt with in the helperServerFree
          * close handler
          */
-        srv->closePipesSafely();
+        comm_close(srv->rfd);
+#ifdef _SQUID_MSWIN_
+
+        if (hIpc) {
+            if (WaitForSingleObject(hIpc, 5000) != WAIT_OBJECT_0) {
+                getCurrentTime();
+                debugs(84, 1, "helperShutdown: WARNING: " << hlp->id_name <<
+                       " #" << no << " (" << hlp->cmdline->key << "," <<
+                       (long int)pid << ") didn't exit in 5 seconds");
+
+            }
+
+            CloseHandle(hIpc);
+        }
+
+#endif
+
     }
 }
 
@@ -633,6 +576,12 @@ helperStatefulShutdown(statefulhelper * hlp)
 {
     dlink_node *link = hlp->servers.head;
     helper_stateful_server *srv;
+#ifdef _SQUID_MSWIN_
+
+    HANDLE hIpc;
+    pid_t pid;
+    int no;
+#endif
 
     while (link) {
         srv = (helper_stateful_server *)link->data;
@@ -643,8 +592,8 @@ helperStatefulShutdown(statefulhelper * hlp)
             continue;
         }
 
-        assert(hlp->childs.n_active > 0);
-        -- hlp->childs.n_active;
+        hlp->n_active--;
+        assert(hlp->n_active >= 0);
         srv->flags.shutdown = 1;	/* request it to shut itself down */
 
         if (srv->flags.busy) {
@@ -666,33 +615,103 @@ helperStatefulShutdown(statefulhelper * hlp)
             }
         }
 
+        srv->flags.closing = 1;
+#ifdef _SQUID_MSWIN_
+
+        hIpc = srv->hIpc;
+        pid = srv->pid;
+        no = srv->index + 1;
+        shutdown(srv->wfd, SD_BOTH);
+#endif
+
         debugs(84, 3, "helperStatefulShutdown: " << hlp->id_name << " #" << srv->index + 1 << " shutting down.");
 
         /* the rest of the details is dealt with in the helperStatefulServerFree
          * close handler
          */
-        srv->closePipesSafely();
+        comm_close(srv->rfd);
+#ifdef _SQUID_MSWIN_
+
+        if (hIpc) {
+            if (WaitForSingleObject(hIpc, 5000) != WAIT_OBJECT_0) {
+                getCurrentTime();
+                debugs(84, 1, "helperShutdown: WARNING: " << hlp->id_name <<
+                       " #" << no << " (" << hlp->cmdline->key << "," <<
+                       (long int)pid << ") didn't exit in 5 seconds");
+            }
+
+            CloseHandle(hIpc);
+        }
+
+#endif
+
     }
 }
 
-helper::~helper()
-{
-    /* note, don't free id_name, it probably points to static memory */
 
-    if (queue.head)
-        debugs(84, 0, "WARNING: freeing " << id_name << " helper with " << stats.queue_size << " requests queued");
+helper *
+helperCreate(const char *name)
+{
+    helper *hlp;
+    CBDATA_INIT_TYPE(helper);
+    hlp = cbdataAlloc(helper);
+    hlp->id_name = name;
+    hlp->eom = '\n';
+    return hlp;
 }
 
+statefulhelper *
+helperStatefulCreate(const char *name)
+{
+    statefulhelper *hlp;
+    CBDATA_INIT_TYPE(statefulhelper);
+    hlp = cbdataAlloc(statefulhelper);
+    hlp->id_name = name;
+    hlp->eom = '\n';
+    return hlp;
+}
+
+
+void
+helperFree(helper * hlp)
+{
+    if (!hlp)
+        return;
+
+    /* note, don't free hlp->name, it probably points to static memory */
+    if (hlp->queue.head)
+        debugs(84, 0, "WARNING: freeing " << hlp->id_name << " helper with " <<
+               hlp->stats.queue_size << " requests queued");
+
+    cbdataFree(hlp);
+}
+
+void
+helperStatefulFree(statefulhelper * hlp)
+{
+    if (!hlp)
+        return;
+
+    /* note, don't free hlp->name, it probably points to static memory */
+    if (hlp->queue.head)
+        debugs(84, 0, "WARNING: freeing " << hlp->id_name << " helper with " <<
+               hlp->stats.queue_size << " requests queued");
+
+    cbdataFree(hlp);
+}
+
+
 /* ====================================================================== */
 /* LOCAL FUNCTIONS */
 /* ====================================================================== */
 
 static void
-helperServerFree(helper_server *srv)
+helperServerFree(int fd, void *data)
 {
+    helper_server *srv = (helper_server *)data;
     helper *hlp = srv->parent;
     helper_request *r;
-    int i, concurrency = hlp->childs.concurrency;
+    int i, concurrency = hlp->concurrency;
 
     if (!concurrency)
         concurrency = 1;
@@ -711,31 +730,34 @@ helperServerFree(helper_server *srv)
         srv->writebuf = NULL;
     }
 
-    if (Comm::IsConnOpen(srv->writePipe))
-        srv->closeWritePipeSafely();
+    if (srv->wfd != srv->rfd && srv->wfd != -1)
+        comm_close(srv->wfd);
 
     dlinkDelete(&srv->link, &hlp->servers);
 
-    assert(hlp->childs.n_running > 0);
-    -- hlp->childs.n_running;
+    hlp->n_running--;
+
+    assert(hlp->n_running >= 0);
 
     if (!srv->flags.shutdown) {
-        assert(hlp->childs.n_active > 0);
-        -- hlp->childs.n_active;
-        debugs(84, DBG_CRITICAL, "WARNING: " << hlp->id_name << " #" << srv->index + 1 << " exited");
+        hlp->n_active--;
+        assert(hlp->n_active >= 0);
+        debugs(84, 0, "WARNING: " << hlp->id_name << " #" << srv->index + 1 <<
+               " (FD " << fd << ") exited");
 
-        if (hlp->childs.needNew() > 0) {
-            debugs(80, 1, "Too few " << hlp->id_name << " processes are running (need " << hlp->childs.needNew() << "/" << hlp->childs.n_max << ")");
+        if (hlp->n_active < hlp->n_to_start / 2) {
+            debugs(80, 0, "Too few " << hlp->id_name << " processes are running");
 
-            if (hlp->childs.n_active < hlp->childs.n_startup && hlp->last_restart > squid_curtime - 30)
+            if (hlp->last_restart > squid_curtime - 30)
                 fatalf("The %s helpers are crashing too rapidly, need help!\n", hlp->id_name);
 
-            debugs(80, 1, "Starting new helpers");
+            debugs(80, 0, "Starting new helpers");
+
             helperOpenServers(hlp);
         }
     }
 
-    for (i = 0; i < concurrency; ++i) {
+    for (i = 0; i < concurrency; i++) {
         if ((r = srv->requests[i])) {
             void *cbdata;
 
@@ -754,8 +776,9 @@ helperServerFree(helper_server *srv)
 }
 
 static void
-helperStatefulServerFree(helper_stateful_server *srv)
+helperStatefulServerFree(int fd, void *data)
 {
+    helper_stateful_server *srv = (helper_stateful_server *)data;
     statefulhelper *hlp = srv->parent;
     helper_stateful_request *r;
 
@@ -772,26 +795,28 @@ helperStatefulServerFree(helper_stateful_server *srv)
 #endif
 
     /* TODO: walk the local queue of requests and carry them all out */
-    if (Comm::IsConnOpen(srv->writePipe))
-        srv->closeWritePipeSafely();
+    if (srv->wfd != srv->rfd && srv->wfd != -1)
+        comm_close(srv->wfd);
 
     dlinkDelete(&srv->link, &hlp->servers);
 
-    assert(hlp->childs.n_running > 0);
-    -- hlp->childs.n_running;
+    hlp->n_running--;
+
+    assert(hlp->n_running >= 0);
 
     if (!srv->flags.shutdown) {
-        assert( hlp->childs.n_active > 0);
-        -- hlp->childs.n_active;
-        debugs(84, 0, "WARNING: " << hlp->id_name << " #" << srv->index + 1 << " exited");
+        hlp->n_active--;
+        assert( hlp->n_active >= 0);
+        debugs(84, 0, "WARNING: " << hlp->id_name << " #" << srv->index + 1 << " (FD " << fd << ") exited");
 
-        if (hlp->childs.needNew() > 0) {
-            debugs(80, 1, "Too few " << hlp->id_name << " processes are running (need " << hlp->childs.needNew() << "/" << hlp->childs.n_max << ")");
+        if (hlp->n_active <= hlp->n_to_start / 2) {
+            debugs(80, 0, "Too few " << hlp->id_name << " processes are running");
 
-            if (hlp->childs.n_active < hlp->childs.n_startup && hlp->last_restart > squid_curtime - 30)
+            if (hlp->last_restart > squid_curtime - 30)
                 fatalf("The %s helpers are crashing too rapidly, need help!\n", hlp->id_name);
 
-            debugs(80, 1, "Starting new helpers");
+            debugs(80, 0, "Starting new helpers");
+
             helperStatefulOpenServers(hlp);
         }
     }
@@ -808,7 +833,7 @@ helperStatefulServerFree(helper_stateful_server *srv)
     }
 
     if (srv->data != NULL)
-        hlp->datapool->freeOne(srv->data);
+        hlp->datapool->free(srv->data);
 
     cbdataReferenceDone(srv->parent);
 
@@ -830,9 +855,9 @@ static void helperReturnBuffer(int request_number, helper_server * srv, helper *
         if (cbdataReferenceValidDone(r->data, &cbdata))
             callback(cbdata, msg);
 
-        -- srv->stats.pending;
+        srv->stats.pending--;
 
-        ++ hlp->stats.replies;
+        hlp->stats.replies++;
 
         srv->answer_time = current_time;
 
@@ -855,14 +880,18 @@ static void helperReturnBuffer(int request_number, helper_server * srv, helper *
     if (!srv->flags.shutdown) {
         helperKickQueue(hlp);
     } else if (!srv->flags.closing && !srv->stats.pending) {
+        int wfd = srv->wfd;
+        srv->wfd = -1;
+        if (srv->rfd == wfd)
+            srv->rfd = -1;
         srv->flags.closing=1;
-        srv->writePipe->close();
+        comm_close(wfd);
         return;
     }
 }
 
 static void
-helperHandleRead(const Comm::ConnectionPointer &conn, char *buf, size_t len, comm_err_t flag, int xerrno, void *data)
+helperHandleRead(int fd, char *buf, size_t len, comm_err_t flag, int xerrno, void *data)
 {
     char *t = NULL;
     helper_server *srv = (helper_server *)data;
@@ -875,12 +904,12 @@ helperHandleRead(const Comm::ConnectionPointer &conn, char *buf, size_t len, com
         return;
     }
 
-    assert(conn->fd == srv->readPipe->fd);
+    assert(fd == srv->rfd);
 
     debugs(84, 5, "helperHandleRead: " << len << " bytes from " << hlp->id_name << " #" << srv->index + 1);
 
     if (flag != COMM_OK || len == 0) {
-        srv->closePipesSafely();
+        comm_close(fd);
         return;
     }
 
@@ -907,49 +936,24 @@ helperHandleRead(const Comm::ConnectionPointer &conn, char *buf, size_t len, com
         if (t > srv->rbuf && t[-1] == '\r' && hlp->eom == '\n')
             t[-1] = '\0';
 
-        *t = '\0';
-        ++t;
+        *t++ = '\0';
 
-        if (hlp->childs.concurrency) {
+        if (hlp->concurrency) {
             i = strtol(msg, &msg, 10);
 
             while (*msg && xisspace(*msg))
-                ++msg;
+                msg++;
         }
 
         helperReturnBuffer(i, srv, hlp, msg, t);
     }
 
-    if (Comm::IsConnOpen(srv->readPipe)) {
-        int spaceSize = srv->rbuf_sz - srv->roffset - 1;
-        assert(spaceSize >= 0);
-
-        // grow the input buffer if needed and possible
-        if (!spaceSize && srv->rbuf_sz + 4096 <= ReadBufMaxSize) {
-            srv->rbuf = (char *)memReallocBuf(srv->rbuf, srv->rbuf_sz + 4096, &srv->rbuf_sz);
-            debugs(84, 3, HERE << "Grew read buffer to " << srv->rbuf_sz);
-            spaceSize = srv->rbuf_sz - srv->roffset - 1;
-            assert(spaceSize >= 0);
-        }
-
-        // quit reading if there is no space left
-        if (!spaceSize) {
-            debugs(84, DBG_IMPORTANT, "ERROR: Disconnecting from a " <<
-                   "helper that overflowed " << srv->rbuf_sz << "-byte " <<
-                   "Squid input buffer: " << hlp->id_name << " #" <<
-                   (srv->index + 1));
-            srv->closePipesSafely();
-            return;
-        }
-
-        AsyncCall::Pointer call = commCbCall(5,4, "helperHandleRead",
-                                             CommIoCbPtrFun(helperHandleRead, srv));
-        comm_read(srv->readPipe, srv->rbuf + srv->roffset, spaceSize, call);
-    }
+    if (srv->rfd != -1)
+        comm_read(fd, srv->rbuf + srv->roffset, srv->rbuf_sz - srv->roffset - 1, helperHandleRead, srv);
 }
 
 static void
-helperStatefulHandleRead(const Comm::ConnectionPointer &conn, char *buf, size_t len, comm_err_t flag, int xerrno, void *data)
+helperStatefulHandleRead(int fd, char *buf, size_t len, comm_err_t flag, int xerrno, void *data)
 {
     char *t = NULL;
     helper_stateful_server *srv = (helper_stateful_server *)data;
@@ -963,14 +967,14 @@ helperStatefulHandleRead(const Comm::ConnectionPointer &conn, char *buf, size_t
         return;
     }
 
-    assert(conn->fd == srv->readPipe->fd);
+    assert(fd == srv->rfd);
 
     debugs(84, 5, "helperStatefulHandleRead: " << len << " bytes from " <<
            hlp->id_name << " #" << srv->index + 1);
 
 
     if (flag != COMM_OK || len == 0) {
-        srv->closePipesSafely();
+        comm_close(fd);
         return;
     }
 
@@ -1008,7 +1012,7 @@ helperStatefulHandleRead(const Comm::ConnectionPointer &conn, char *buf, size_t
         srv->roffset = 0;
         helperStatefulRequestFree(r);
         srv->request = NULL;
-        ++ hlp->stats.replies;
+        hlp->stats.replies++;
         srv->answer_time = current_time;
         hlp->stats.avg_svc_time =
             Math::intAverage(hlp->stats.avg_svc_time,
@@ -1021,32 +1025,9 @@ helperStatefulHandleRead(const Comm::ConnectionPointer &conn, char *buf, size_t
             helperStatefulReleaseServer(srv);
     }
 
-    if (Comm::IsConnOpen(srv->readPipe)) {
-        int spaceSize = srv->rbuf_sz - srv->roffset - 1;
-        assert(spaceSize >= 0);
-
-        // grow the input buffer if needed and possible
-        if (!spaceSize && srv->rbuf_sz + 4096 <= ReadBufMaxSize) {
-            srv->rbuf = (char *)memReallocBuf(srv->rbuf, srv->rbuf_sz + 4096, &srv->rbuf_sz);
-            debugs(84, 3, HERE << "Grew read buffer to " << srv->rbuf_sz);
-            spaceSize = srv->rbuf_sz - srv->roffset - 1;
-            assert(spaceSize >= 0);
-        }
-
-        // quit reading if there is no space left
-        if (!spaceSize) {
-            debugs(84, DBG_IMPORTANT, "ERROR: Disconnecting from a " <<
-                   "helper that overflowed " << srv->rbuf_sz << "-byte " <<
-                   "Squid input buffer: " << hlp->id_name << " #" <<
-                   (srv->index + 1));
-            srv->closePipesSafely();
-            return;
-        }
-
-        AsyncCall::Pointer call = commCbCall(5,4, "helperStatefulHandleRead",
-                                             CommIoCbPtrFun(helperStatefulHandleRead, srv));
-        comm_read(srv->readPipe, srv->rbuf + srv->roffset, spaceSize, call);
-    }
+    if (srv->rfd != -1)
+        comm_read(srv->rfd, srv->rbuf + srv->roffset, srv->rbuf_sz - srv->roffset - 1,
+                  helperStatefulHandleRead, srv);
 }
 
 static void
@@ -1054,16 +1035,9 @@ Enqueue(helper * hlp, helper_request * r)
 {
     dlink_node *link = (dlink_node *)memAllocate(MEM_DLINK_NODE);
     dlinkAddTail(r, link, &hlp->queue);
-    ++ hlp->stats.queue_size;
+    hlp->stats.queue_size++;
 
-    /* do this first so idle=N has a chance to grow the child pool before it hits critical. */
-    if (hlp->childs.needNew() > 0) {
-        debugs(84, 0, "Starting new " << hlp->id_name << " helpers...");
-        helperOpenServers(hlp);
-        return;
-    }
-
-    if (hlp->stats.queue_size < (int)hlp->childs.n_running)
+    if (hlp->stats.queue_size < hlp->n_running)
         return;
 
     if (squid_curtime - hlp->last_queue_warn < 600)
@@ -1074,12 +1048,15 @@ Enqueue(helper * hlp, helper_request * r)
 
     hlp->last_queue_warn = squid_curtime;
 
-    debugs(84, 0, "WARNING: All " << hlp->childs.n_active << "/" << hlp->childs.n_max << " " << hlp->id_name << " processes are busy.");
+    debugs(84, 0, "WARNING: All " << hlp->id_name << " processes are busy.");
     debugs(84, 0, "WARNING: " << hlp->stats.queue_size << " pending requests queued");
-    debugs(84, 0, "WARNING: Consider increasing the number of " << hlp->id_name << " processes in your config file.");
 
-    if (hlp->stats.queue_size > (int)hlp->childs.n_running * 2)
+
+    if (hlp->stats.queue_size > hlp->n_running * 2)
         fatalf("Too many queued %s requests", hlp->id_name);
+
+    debugs(84, 1, "Consider increasing the number of " << hlp->id_name << " processes in your config file.");
+
 }
 
 static void
@@ -1087,19 +1064,12 @@ StatefulEnqueue(statefulhelper * hlp, helper_stateful_request * r)
 {
     dlink_node *link = (dlink_node *)memAllocate(MEM_DLINK_NODE);
     dlinkAddTail(r, link, &hlp->queue);
-    ++ hlp->stats.queue_size;
+    hlp->stats.queue_size++;
 
-    /* do this first so idle=N has a chance to grow the child pool before it hits critical. */
-    if (hlp->childs.needNew() > 0) {
-        debugs(84, 0, "Starting new " << hlp->id_name << " helpers...");
-        helperStatefulOpenServers(hlp);
-        return;
-    }
-
-    if (hlp->stats.queue_size < (int)hlp->childs.n_running)
+    if (hlp->stats.queue_size < hlp->n_running)
         return;
 
-    if (hlp->stats.queue_size > (int)hlp->childs.n_running * 2)
+    if (hlp->stats.queue_size > hlp->n_running * 2)
         fatalf("Too many queued %s requests", hlp->id_name);
 
     if (squid_curtime - hlp->last_queue_warn < 600)
@@ -1110,9 +1080,11 @@ StatefulEnqueue(statefulhelper * hlp, helper_stateful_request * r)
 
     hlp->last_queue_warn = squid_curtime;
 
-    debugs(84, 0, "WARNING: All " << hlp->childs.n_active << "/" << hlp->childs.n_max << " " << hlp->id_name << " processes are busy.");
+    debugs(84, 0, "WARNING: All " << hlp->id_name << " processes are busy.");
+
     debugs(84, 0, "WARNING: " << hlp->stats.queue_size << " pending requests queued");
-    debugs(84, 0, "WARNING: Consider increasing the number of " << hlp->id_name << " processes in your config file.");
+    debugs(84, 1, "Consider increasing the number of " << hlp->id_name << " processes in your config file.");
+
 }
 
 static helper_request *
@@ -1125,7 +1097,7 @@ Dequeue(helper * hlp)
         r = (helper_request *)link->data;
         dlinkDelete(link, &hlp->queue);
         memFree(link, MEM_DLINK_NODE);
-        -- hlp->stats.queue_size;
+        hlp->stats.queue_size--;
     }
 
     return r;
@@ -1141,7 +1113,7 @@ StatefulDequeue(statefulhelper * hlp)
         r = (helper_stateful_request *)link->data;
         dlinkDelete(link, &hlp->queue);
         memFree(link, MEM_DLINK_NODE);
-        -- hlp->stats.queue_size;
+        hlp->stats.queue_size--;
     }
 
     return r;
@@ -1153,9 +1125,8 @@ GetFirstAvailable(helper * hlp)
     dlink_node *n;
     helper_server *srv;
     helper_server *selected = NULL;
-    debugs(84, 5, "GetFirstAvailable: Running servers " << hlp->childs.n_running);
 
-    if (hlp->childs.n_running == 0)
+    if (hlp->n_running == 0)
         return NULL;
 
     /* Find "least" loaded helper (approx) */
@@ -1180,17 +1151,12 @@ GetFirstAvailable(helper * hlp)
     }
 
     /* Check for overload */
-    if (!selected) {
-        debugs(84, 5, "GetFirstAvailable: None available.");
+    if (!selected)
         return NULL;
-    }
 
-    if (selected->stats.pending >= (hlp->childs.concurrency ? hlp->childs.concurrency : 1)) {
-        debugs(84, 3, "GetFirstAvailable: Least-loaded helper is overloaded!");
+    if (selected->stats.pending >= (hlp->concurrency ? hlp->concurrency : 1))
         return NULL;
-    }
 
-    debugs(84, 5, "GetFirstAvailable: returning srv-" << selected->index);
     return selected;
 }
 
@@ -1199,9 +1165,9 @@ StatefulGetFirstAvailable(statefulhelper * hlp)
 {
     dlink_node *n;
     helper_stateful_server *srv = NULL;
-    debugs(84, 5, "StatefulGetFirstAvailable: Running servers " << hlp->childs.n_running);
+    debugs(84, 5, "StatefulGetFirstAvailable: Running servers " << hlp->n_running);
 
-    if (hlp->childs.n_running == 0)
+    if (hlp->n_running == 0)
         return NULL;
 
     for (n = hlp->servers.head; n != NULL; n = n->next) {
@@ -1229,7 +1195,7 @@ StatefulGetFirstAvailable(statefulhelper * hlp)
 
 
 static void
-helperDispatchWriteDone(const Comm::ConnectionPointer &conn, char *buf, size_t len, comm_err_t flag, int xerrno, void *data)
+helperDispatchWriteDone(int fd, char *buf, size_t len, comm_err_t flag, int xerrno, void *data)
 {
     helper_server *srv = (helper_server *)data;
 
@@ -1248,9 +1214,11 @@ helperDispatchWriteDone(const Comm::ConnectionPointer &conn, char *buf, size_t l
         srv->writebuf = srv->wqueue;
         srv->wqueue = new MemBuf;
         srv->flags.writing = 1;
-        AsyncCall::Pointer call = commCbCall(5,5, "helperDispatchWriteDone",
-                                             CommIoCbPtrFun(helperDispatchWriteDone, srv));
-        Comm::Write(srv->writePipe, srv->writebuf->content(), srv->writebuf->contentSize(), call, NULL);
+        comm_write(srv->wfd,
+                   srv->writebuf->content(),
+                   srv->writebuf->contentSize(),
+                   helperDispatchWriteDone,	/* Handler */
+                   srv, NULL);			/* Handler-data, freefunc */
     }
 }
 
@@ -1267,7 +1235,7 @@ helperDispatch(helper_server * srv, helper_request * r)
         return;
     }
 
-    for (slot = 0; slot < (hlp->childs.concurrency ? hlp->childs.concurrency : 1); ++slot) {
+    for (slot = 0; slot < (hlp->concurrency ? hlp->concurrency : 1); slot++) {
         if (!srv->requests[slot]) {
             ptr = &srv->requests[slot];
             break;
@@ -1282,7 +1250,7 @@ helperDispatch(helper_server * srv, helper_request * r)
     if (srv->wqueue->isNull())
         srv->wqueue->init();
 
-    if (hlp->childs.concurrency)
+    if (hlp->concurrency)
         srv->wqueue->Printf("%d %s", slot, r->buf);
     else
         srv->wqueue->append(r->buf, strlen(r->buf));
@@ -1292,19 +1260,21 @@ helperDispatch(helper_server * srv, helper_request * r)
         srv->writebuf = srv->wqueue;
         srv->wqueue = new MemBuf;
         srv->flags.writing = 1;
-        AsyncCall::Pointer call = commCbCall(5,5, "helperDispatchWriteDone",
-                                             CommIoCbPtrFun(helperDispatchWriteDone, srv));
-        Comm::Write(srv->writePipe, srv->writebuf->content(), srv->writebuf->contentSize(), call, NULL);
+        comm_write(srv->wfd,
+                   srv->writebuf->content(),
+                   srv->writebuf->contentSize(),
+                   helperDispatchWriteDone,	/* Handler */
+                   srv, NULL);			/* Handler-data, free func */
     }
 
     debugs(84, 5, "helperDispatch: Request sent to " << hlp->id_name << " #" << srv->index + 1 << ", " << strlen(r->buf) << " bytes");
 
-    ++ srv->stats.uses;
-    ++ hlp->stats.requests;
+    srv->stats.uses++;
+    hlp->stats.requests++;
 }
 
 static void
-helperStatefulDispatchWriteDone(const Comm::ConnectionPointer &conn, char *buf, size_t len, comm_err_t flag,
+helperStatefulDispatchWriteDone(int fd, char *buf, size_t len, comm_err_t flag,
                                 int xerrno, void *data)
 {
     /* nothing! */
@@ -1345,15 +1315,17 @@ helperStatefulDispatch(helper_stateful_server * srv, helper_stateful_request * r
     srv->flags.reserved = 1;
     srv->request = r;
     srv->dispatch_time = current_time;
-    AsyncCall::Pointer call = commCbCall(5,5, "helperStatefulDispatchWriteDone",
-                                         CommIoCbPtrFun(helperStatefulDispatchWriteDone, hlp));
-    Comm::Write(srv->writePipe, r->buf, strlen(r->buf), call, NULL);
+    comm_write(srv->wfd,
+               r->buf,
+               strlen(r->buf),
+               helperStatefulDispatchWriteDone,	/* Handler */
+               hlp, NULL);				/* Handler-data, free func */
     debugs(84, 5, "helperStatefulDispatch: Request sent to " <<
            hlp->id_name << " #" << srv->index + 1 << ", " <<
            (int) strlen(r->buf) << " bytes");
 
-    ++ srv->stats.uses;
-    ++ hlp->stats.requests;
+    srv->stats.uses++;
+    hlp->stats.requests++;
 }
 
 
@@ -1383,7 +1355,12 @@ helperStatefulServerDone(helper_stateful_server * srv)
     if (!srv->flags.shutdown) {
         helperStatefulKickQueue(srv->parent);
     } else if (!srv->flags.closing && !srv->flags.reserved && !srv->flags.busy) {
-        srv->closeWritePipeSafely();
+        int wfd = srv->wfd;
+        srv->wfd = -1;
+        if (srv->rfd == wfd)
+            srv->rfd = -1;
+        srv->flags.closing=1;
+        comm_close(wfd);
         return;
     }
 }
