@@ -34,11 +34,8 @@
  */
 
 #include "squid.h"
-#if USE_AUTH
 #include "auth/UserRequest.h"
-#endif
-#include "comm/Connection.h"
-#include "mgr/Registration.h"
+#include "CacheManager.h"
 #include "Store.h"
 #include "fde.h"
 #include "client_side_request.h"
@@ -48,9 +45,6 @@
 #include "client_side_reply.h"
 #include "helper.h"
 #include "rfc1738.h"
-#if USE_SSL
-#include "ssl/support.h"
-#endif
 
 /// url maximum lengh + extra informations passed to redirector
 #define MAX_REDIRECTOR_REQUEST_STRLEN (MAX_URL + 1024)
@@ -59,7 +53,7 @@ typedef struct {
     void *data;
     char *orig_url;
 
-    Ip::Address client_addr;
+    IpAddress client_addr;
     const char *client_ident;
     const char *method_s;
     RH *handler;
@@ -147,30 +141,21 @@ redirectStart(ClientHttpRequest * http, RH * handler, void *data)
     else
         r->client_addr.SetNoAddr();
     r->client_ident = NULL;
-#if USE_AUTH
-    if (http->request->auth_user_request != NULL) {
+
+    if (http->request->auth_user_request)
         r->client_ident = http->request->auth_user_request->username();
-        debugs(61, 5, HERE << "auth-user=" << (r->client_ident?r->client_ident:"NULL"));
-    }
-#endif
-
-    // HttpRequest initializes with null_string. So we must check both defined() and size()
-    if (!r->client_ident && http->request->extacl_user.defined() && http->request->extacl_user.size()) {
+    else if (http->request->extacl_user.defined()) {
         r->client_ident = http->request->extacl_user.termedBuf();
-        debugs(61, 5, HERE << "acl-user=" << (r->client_ident?r->client_ident:"NULL"));
     }
 
-    if (!r->client_ident && conn != NULL && conn->clientConnection != NULL && conn->clientConnection->rfc931[0]) {
-        r->client_ident = conn->clientConnection->rfc931;
-        debugs(61, 5, HERE << "ident-user=" << (r->client_ident?r->client_ident:"NULL"));
-    }
+    if (!r->client_ident && (conn != NULL && conn->rfc931[0]))
+        r->client_ident = conn->rfc931;
 
 #if USE_SSL
 
-    if (!r->client_ident && conn != NULL && Comm::IsConnOpen(conn->clientConnection)) {
-        r->client_ident = sslGetUserEmail(fd_table[conn->clientConnection->fd].ssl);
-        debugs(61, 5, HERE << "ssl-user=" << (r->client_ident?r->client_ident:"NULL"));
-    }
+    if (!r->client_ident && conn != NULL)
+        r->client_ident = sslGetUserEmail(fd_table[conn->fd].ssl);
+
 #endif
 
     if (!r->client_ident)
@@ -206,20 +191,15 @@ redirectStart(ClientHttpRequest * http, RH * handler, void *data)
         clientStreamNode *node = (clientStreamNode *)http->client_stream.tail->prev->data;
         clientReplyContext *repContext = dynamic_cast<clientReplyContext *>(node->data.getRaw());
         assert (repContext);
-        Ip::Address tmpnoaddr;
+        IpAddress tmpnoaddr;
         tmpnoaddr.SetNoAddr();
         repContext->setReplyToError(ERR_GATEWAY_FAILURE, status,
                                     http->request->method, NULL,
-                                    http->getConn() != NULL && http->getConn()->clientConnection != NULL ?
-                                    http->getConn()->clientConnection->remote : tmpnoaddr,
+                                    http->getConn() != NULL ? http->getConn()->peer : tmpnoaddr,
                                     http->request,
                                     NULL,
-#if USE_AUTH
                                     http->getConn() != NULL && http->getConn()->auth_user_request != NULL ?
                                     http->getConn()->auth_user_request : http->request->auth_user_request);
-#else
-                                    NULL);
-#endif
 
         node = (clientStreamNode *)http->client_stream.tail->data;
         clientStreamRead(node, http, node->readBuffer);
@@ -233,7 +213,8 @@ redirectStart(ClientHttpRequest * http, RH * handler, void *data)
 static void
 redirectRegisterWithCacheManager(void)
 {
-    Mgr::RegisterAction("redirector", "URL Redirector Stats", redirectStats, 0, 1);
+    CacheManager::GetInstance()->
+    registerAction("redirector", "URL Redirector Stats", redirectStats, 0, 1);
 }
 
 void
@@ -247,11 +228,13 @@ redirectInit(void)
         return;
 
     if (redirectors == NULL)
-        redirectors = new helper("redirector");
+        redirectors = helperCreate("redirector");
 
     redirectors->cmdline = Config.Program.redirect;
 
-    redirectors->childs.updateLimits(Config.redirectChildren);
+    redirectors->n_to_start = Config.redirectChildren;
+
+    redirectors->concurrency = Config.redirectConcurrency;
 
     redirectors->ipc_type = IPC_STREAM;
 
@@ -274,6 +257,7 @@ redirectShutdown(void)
     if (!shutting_down)
         return;
 
-    delete redirectors;
+    helperFree(redirectors);
+
     redirectors = NULL;
 }
